!---------------------------------------------------------------------- 
!---------------------------------------------------------------------- 
!   brt :    A boundary value problem (Bratu's equation)
!---------------------------------------------------------------------- 
!---------------------------------------------------------------------- 

      SUBROUTINE FUNC(NDIM,U,ICP,PAR,IJAC,F,DFDU,DFDP) 
!     ---------- ---- 

      IMPLICIT NONE
      INTEGER, INTENT(IN) :: NDIM, ICP(*), IJAC
      DOUBLE PRECISION, INTENT(IN) :: U(NDIM), PAR(*)
      DOUBLE PRECISION, INTENT(OUT) :: F(NDIM)
      DOUBLE PRECISION, INTENT(INOUT) :: DFDU(NDIM,NDIM), DFDP(NDIM,*)

      DOUBLE PRECISION F1, F2, H, z
      DOUBLE PRECISION pi, lambda, eps, uhat
      DOUBLE PRECISION u1, u2, w1, w2

! The RHS is defined by these two functions:
       F1(z) =-lambda*DEXP(z)		! if z <= uhat
       F2(z) =-lambda*DEXP(uhat)	! if z >  uhat

! The approximate Heaviside function:
       H(z)  = 0.5d0 + (1.d0/pi)*DATAN(z/eps) 

       pi = 4*DATAN(1.d0)
       lambda = PAR(1)
       eps    = PAR(2)
       uhat   = PAR(3)

       u1 = U(1)
       u2 = U(2)

       w1 = H(uhat-u1)		! The "weight" of F1
       w2 = 1.d0 - w1		! The "weight" of F2

       F(1) = u2
       F(2) = w1*F1(u1) + w2*F2(u1)

      END SUBROUTINE FUNC
!---------------------------------------------------------------------- 

      SUBROUTINE STPNT(NDIM,U,PAR,x) 
!     ---------- ----- 

      IMPLICIT NONE
      INTEGER, INTENT(IN) :: NDIM
      DOUBLE PRECISION, INTENT(INOUT) :: U(NDIM),PAR(*)
      DOUBLE PRECISION, INTENT(IN) :: x

      DOUBLE PRECISION lambda, eps, uhat

       lambda=0.d0 	! The starting value of lambda
       eps   = 1e-5	! The smoothing parameter (reset in brs.auto)
       uhat  = 2.d0	! The critical value of u (reset in brs.auto)

       PAR(1) = lambda
       PAR(2) = eps
       PAR(3) = uhat

!      The constant starting solution:
       U(1)=0.d0
       U(2)=0.d0

      END SUBROUTINE STPNT
!---------------------------------------------------------------------- 

      SUBROUTINE BCND(NDIM,PAR,ICP,NBC,U0,U1,FB,IJAC,DBC) 
!     ---------- ---- 

      IMPLICIT NONE
      INTEGER, INTENT(IN) :: NDIM, ICP(*), NBC, IJAC
      DOUBLE PRECISION, INTENT(IN) :: PAR(*), U0(NDIM), U1(NDIM)
      DOUBLE PRECISION, INTENT(OUT) :: FB(NBC)
      DOUBLE PRECISION, INTENT(INOUT) :: DBC(NBC,*)

       FB(1)=U0(1)	! u(x)=0 at x=0 
       FB(2)=U1(1)	! u(x)=0 at x=1

      END SUBROUTINE BCND
!---------------------------------------------------------------------- 
      SUBROUTINE PVLS
      END SUBROUTINE PVLS

      SUBROUTINE ICND
      END SUBROUTINE ICND

      SUBROUTINE FOPT 
      END SUBROUTINE FOPT
!---------------------------------------------------------------------- 
