!---------------------------------------------------------------------- 
!---------------------------------------------------------------------- 
!       Time integration of the Gelfand-Bratu equation
!---------------------------------------------------------------------- 
!---------------------------------------------------------------------- 

  SUBROUTINE FUNC(NDIM,U,ICP,PAR,IJAC,F,DFDU,DFDP) 
! ---------- ---- 

  IMPLICIT NONE
  INTEGER, INTENT(IN) :: NDIM, ICP(*), IJAC
  DOUBLE PRECISION, INTENT(IN) :: U(2*NDIM+1), PAR(*)
  DOUBLE PRECISION, INTENT(OUT) :: F(NDIM)
  DOUBLE PRECISION, INTENT(INOUT) :: DFDU(NDIM,NDIM), DFDP(NDIM,*)
  DOUBLE PRECISION lambda, t0, t1, dt, u1, v1, u0, v0, ut

   lambda = PAR(1)

   t0 = PAR(13)		! The "old" time
   t1 = PAR(14)		! The current time

   u1 = U(1)		! The current u
   v1 = U(2)		! The current v

! Note that UOLD is not an argument in the call to FUNC, because
! UOLD is rarely needed here. However, UOLD can be accessed, since
! it is located immediately following U, namely, in U(NDIM+1-2NDIM).

   u0 = U(3)		! UOLD(1): u at the preceding time step
   v0 = U(4)		! UOLD(2): v at the preceding time step

! The time derivative:

   dt = t1-t0
   IF (dt .EQ. 0.d0) THEN
     dt = 1.d0		! A harmless fix
   ENDIF
   ut = (u1-u0)/dt

! The PDE as a first order system in the space-variable x:

   F(1)= v1
   F(2)= ut - lambda*DEXP(u1)

   IF(IJAC==0)RETURN

! It is advisable to specify the Jacobian here and in BCND:

   DFDU(1,1) = 0
   DFDU(1,2) = 1

   DFDU(2,1) = 1/dt-lambda*EXP(u1)
   DFDU(2,2) = 0

   IF(IJAC==1)RETURN

   DFDP(1,14) = 0.d0
   DFDP(2,14) =-(u1-u0)/(dt**2)

  END SUBROUTINE FUNC
!---------------------------------------------------------------------- 

  SUBROUTINE STPNT(NDIM,U,PAR,x) 
! ---------- ----- 

  IMPLICIT NONE
  INTEGER, INTENT(IN) :: NDIM
  DOUBLE PRECISION, INTENT(INOUT) :: U(NDIM),PAR(*)
  DOUBLE PRECISION, INTENT(IN) :: x
  DOUBLE PRECISION pi, lambda, amp, t0, t1

   lambda = 1.d0
   amp = 0.1d0
   t0 = 0.d0
   t1 = 0.d0

   PAR(1)  = lambda
   PAR(2)  = amp
   PAR(13) = t0
   PAR(14) = t1

   pi   = 4*ATAN(1.d0)
   U(1) = amp*SIN(pi*x)
   U(2) = amp*pi*COS(pi*x)

  END SUBROUTINE STPNT
!---------------------------------------------------------------------- 

  SUBROUTINE BCND(NDIM,PAR,ICP,NBC,U0,U1,FB,IJAC,DBC) 
! ---------- ---- 

  IMPLICIT NONE
  INTEGER, INTENT(IN) :: NDIM, ICP(*), NBC, IJAC
  DOUBLE PRECISION, INTENT(IN) :: PAR(*), U0(NDIM), U1(NDIM)
  DOUBLE PRECISION, INTENT(OUT) :: FB(NBC)
  DOUBLE PRECISION, INTENT(INOUT) :: DBC(NBC,*)

   FB(1) = U0(1) 
   FB(2) = U1(1) 

  IF(IJAC==0)RETURN

   DBC(1,1) = 1
   DBC(2,3) = 1

  END SUBROUTINE BCND
!---------------------------------------------------------------------- 

  SUBROUTINE PVLS(NDIM,U,PAR)
! ---------- ----

  IMPLICIT NONE
  INTEGER, INTENT(IN) :: NDIM
  DOUBLE PRECISION, INTENT(IN) :: U(NDIM)
  DOUBLE PRECISION, INTENT(INOUT) :: PAR(*)
  DOUBLE PRECISION GETP

! Put the maximum of v in PAR(8):
   PAR(8) = GETP('MAX',2,U)

! Put the integral of u in PAR(9):
   PAR(9)=GETP('INT',1,U)

! Copy the "current time" in PAR(14) to the "old time" in PAR(13):
   PAR(13) = PAR(14)

  END SUBROUTINE PVLS
!---------------------------------------------------------------------- 

  SUBROUTINE ICND
  END SUBROUTINE ICND

  SUBROUTINE FOPT 
  END SUBROUTINE FOPT
!---------------------------------------------------------------------- 
