!---------------------------------------------------------------------- 
!---------------------------------------------------------------------- 
!       Time integration of the nonlinear parabolic Bratu equation
!---------------------------------------------------------------------- 
!---------------------------------------------------------------------- 

 SUBROUTINE FUNC(NDIM,U,ICP,PAR,IJAC,F,DFDU,DFDP) 
!     ---------- ---- 

 IMPLICIT NONE
 INTEGER, INTENT(IN) :: NDIM, ICP(*), IJAC
 DOUBLE PRECISION, INTENT(IN) :: U(NDIM), PAR(*)
 DOUBLE PRECISION, INTENT(OUT) :: F(NDIM)
 DOUBLE PRECISION, INTENT(INOUT) :: DFDU(NDIM,NDIM), DFDP(NDIM,*)

! Set the nonlinear term
  F(1)= PAR(1) * EXP(U(1))

 END SUBROUTINE FUNC
!---------------------------------------------------------------------- 

 SUBROUTINE STPNT(NDIM,U,PAR,x) 
!---------- ----- 

 IMPLICIT NONE
 INTEGER, INTENT(IN) :: NDIM
 DOUBLE PRECISION, INTENT(INOUT) :: U(NDIM),PAR(*)
 DOUBLE PRECISION, INTENT(IN) :: x
 DOUBLE PRECISION pi, lambda, amp, L

  pi = 4*ATAN(1.d0)
  lambda = 1.d0	        !the value of lambda, which is reset in brt.auto
  amp    = 0.1d0	!the amplitude of the initial condition
  L      = 1.d0		!the size of the space interval

  PAR(1) = lambda
  PAR(2) = amp
  PAR(11)= L

! The initial data at time 0:
  U(1) = PAR(2)*SIN(pi*x)

! The derivative of U(1) with respect to x must also be given.
! Note the scaling by 1/L:
  U(2) = amp*pi*COS(pi*x)/L

  PAR(15) = 1.d0	!the diffusion constant

 END SUBROUTINE STPNT
!---------------------------------------------------------------------- 

 SUBROUTINE BCND(NDIM,PAR,ICP,NBC,U0,U1,FB,IJAC,DBC) 
!---------- ---- 

 IMPLICIT NONE
 INTEGER, INTENT(IN) :: NDIM, ICP(*), NBC, IJAC
 DOUBLE PRECISION, INTENT(IN) :: PAR(*), U0(NDIM), U1(NDIM)
 DOUBLE PRECISION, INTENT(OUT) :: FB(NBC)
 DOUBLE PRECISION, INTENT(INOUT) :: DBC(NBC,*)

  FB(1) = U0(1) 
  FB(2) = U1(1) 

 END SUBROUTINE BCND
!---------------------------------------------------------------------- 
 SUBROUTINE PVLS(NDIM,U,PAR)
!     ---------- ----

 IMPLICIT NONE
 INTEGER, INTENT(IN) :: NDIM
 DOUBLE PRECISION, INTENT(IN) :: U(NDIM)
 DOUBLE PRECISION, INTENT(INOUT) :: PAR(*)

 DOUBLE PRECISION, EXTERNAL :: GETP,GETU2
 INTEGER NDX,NCOL,NTST

! Put the maximum of v in PAR(8):
   PAR(8) = GETP('MAX',2,U)

! Put the integral of u in PAR(9):
   PAR(9)=GETP('INT',1,U)

 END SUBROUTINE PVLS
!---------------------------------------------------------------------- 
 SUBROUTINE ICND
 END SUBROUTINE ICND

 SUBROUTINE FOPT 
 END SUBROUTINE FOPT
!---------------------------------------------------------------------- 
